#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEServer.h>
#include <Adafruit_NeoPixel.h>
#include <Arduino.h>
#include <Preferences.h>
#include <Wire.h>
#include <Adafruit_MPR121.h>

Preferences preferences;

// Create the sensor object
Adafruit_MPR121 cap = Adafruit_MPR121();
String Gesture;
int currentXPosition = 0;  // Track the current value of x position
int currentYPosition = 0;  // Track the current value of y position
int previousXPosition = 0; // Track the previous value of x position
int previousYPosition = 0; // Track the previous value of y position

unsigned long lastTapTime = 0; // Time of the last tap
const unsigned long doubleTapDelay = 400; // Maximum delay between taps for double tap detection (milliseconds)

//Button
int Mode = 0;
int flag = 0;

//Neopixel
int PIN  = 2;
#define NUMPIXELS 1
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

//Bt DATA
String redBT;
String greenBT;
String blueBT;
String modeBT;
int change = 1;
int modeS = 0;

//DATA
int Data[3][3] = {
  {255, 0, 0},  // Mode 1 {R, G, B}
  {0, 255, 0},  // Mode 2
  {0, 0, 255}   // Mode 3
};

const int rows = 3;
const int cols = 3;

//Bluetooth
#define SERVICE_UUID        "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID "beb5483e-36e1-4688-b7f5-ea07361b26a8"

BLECharacteristic *pCharacteristic; // Declare pCharacteristic as a global variable

// Function that saves data
void saveData() {
  preferences.begin("matrix", false);
  for (int i = 0; i < rows; i++) {
    for (int j = 0; j < cols; j++) {
      String key = "cell" + String(i) + "_" + String(j);
      preferences.putInt(key.c_str(), Data[i][j]);
    }
  }
  preferences.end();
}

void loadData() {
  Serial.println("Loading Data...");
  preferences.begin("matrix", true);
  for (int i = 0; i < rows; i++) {
    for (int j = 0; j < cols; j++) {
      String key = "cell" + String(i) + "_" + String(j);
      Data[i][j] = preferences.getInt(key.c_str(), 0); // Default 0 if not exists
      Serial.println(Data[i][j]);
    }
  }
  preferences.end();
}

class MyCallbacks : public BLECharacteristicCallbacks {
  void onWrite(BLECharacteristic *pCharacteristic) {
    String value = pCharacteristic->getValue();
    redBT = "";
    greenBT = "";
    blueBT = "";
    if (value.length() > 0) {
      Serial.println("*********");
      Serial.print("New value: ");
      for (int i = 0; i < value.length(); i++) {
        // MODES
        if (value[i] == 'A') {
          modeS = 0;
        }
        if (value[i] == 'B') {
          modeS = 1;
        }
        if (value[i] == 'C') {
          modeS = 2;
        }

        // RGB
        if (value[i] == ',') {
          change = change + 1;
        }
        if (change == 1 && isdigit(value[i])) {
          redBT = redBT + value[i];
        }
        else if (change == 2 && isdigit(value[i])) {
          greenBT = greenBT + value[i];
        }
        else if (change == 3 && isdigit(value[i])) {
          blueBT = blueBT + value[i];
        }
      }

      Data[modeS][0] = redBT.toInt();
      Data[modeS][1] = greenBT.toInt();
      Data[modeS][2] = blueBT.toInt();

      Serial.print("Mode = ");
      Serial.println(modeS);
      Serial.println();
      Serial.println("LED COLOR");
      Serial.print("Red = ");
      Serial.println(Data[modeS][0]);
      Serial.print("Green = ");
      Serial.println(Data[modeS][1]);
      Serial.print("Blue = ");
      Serial.println(Data[modeS][2]);
      Serial.println();
      Serial.println("*********");
      change = 1;
      modeS = 0;

      saveData();
    }
  }
};

void setup() {
  // Serial
  Serial.begin(115200);

  Serial.println("Setting up");
  preferences.begin("my-app", false);

  loadData();

  // Bt
  BLEDevice::init("AMEC");
  BLEServer *pServer = BLEDevice::createServer();

  BLEService *pService = pServer->createService(SERVICE_UUID);

  pCharacteristic = pService->createCharacteristic(CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_READ | BLECharacteristic::PROPERTY_WRITE | BLECharacteristic::PROPERTY_NOTIFY);

  pCharacteristic->setCallbacks(new MyCallbacks());

  pCharacteristic->setValue("Hello World");
  pService->start();

  BLEAdvertising *pAdvertising = pServer->getAdvertising();
  pAdvertising->start();

  // Initialize Pixel
  pixels.begin();
  pixels.clear();
  pixels.show();

  // Button
  pinMode(D6, INPUT);

  // Initialize the sensor
  if (!cap.begin(0x5A)) {  // Default I2C address for MPR121 is 0x5A
    while (1);
  }
}

void loop() {
  // Get the currently touched pads
  int touched = cap.touched();

  // Reset current positions
  currentXPosition = 0;
  currentYPosition = 0;

  // Check each pad to see if it is currently touched
  for (int i = 0; i < 12; i++) {
    if (touched & (1 << i)) {
      if (i < 6) {  // Pins 0-5
        currentXPosition = i + 1;  // Save the position (1-based)
      } else {  // Pins 6-11
        currentYPosition = (i - 6) + 1;  // Save the position (1-based)
      }
    }
  }

  if (digitalRead(D6) == HIGH) {
    if (Mode < 2 && flag == 0) {
      flag = 1;
      Mode = Mode + 1;
      Serial.println("Button pressed");
    }
    if (Mode == 2 && flag == 0) {
      Mode = -1;
    }
  }
  else if (digitalRead(D6) == LOW) {
    flag = 0;
  }

  if (Mode == 0) {
    pixels.clear();
    pixels.setPixelColor(0, pixels.Color(Data[0][0], Data[0][1], Data[0][2]));
    pixels.show();
  }
  else if (Mode == 1) {
    pixels.clear();
    pixels.setPixelColor(0, pixels.Color(Data[1][0], Data[1][1], Data[1][2]));
    pixels.show();
  }
  else if (Mode == 2) {
    pixels.clear();
    pixels.setPixelColor(0, pixels.Color(Data[2][0], Data[2][1], Data[2][2]));
    pixels.show();
  }

  // Detect gestures
  if (currentXPosition != 0 || currentYPosition != 0) {
    // Detect double tap
    unsigned long currentTime = millis();
  
    // Detect swipe gestures
    if (previousXPosition != 0 && previousYPosition != 0) {
      if (currentXPosition == previousXPosition) {
        if (currentYPosition < previousYPosition) {
          Serial.println("Swipe Up Detected");
          Gesture="SU";
          pCharacteristic->setValue(Gesture.c_str());  // Convert std::string to C-style string
          pCharacteristic->notify();
          delay(2000);
        } else if (currentYPosition > previousYPosition) {
          Serial.println("Swipe Down Detected");
          Gesture="SD";
          pCharacteristic->setValue(Gesture.c_str());  // Convert std::string to C-style string
          pCharacteristic->notify();
          delay(2000);
        }
      } else if (currentYPosition == previousYPosition) {
        if (currentXPosition < previousXPosition) {
          Serial.println("Swipe Left Detected");
          Gesture="SL";
          pCharacteristic->setValue(Gesture.c_str());  // Convert std::string to C-style string
          pCharacteristic->notify();
          delay(2000);
        } else if (currentXPosition > previousXPosition) {
          Serial.println("Swipe Right Detected");
          Gesture="SR";
          pCharacteristic->setValue(Gesture.c_str());  // Convert std::string to C-style string
          pCharacteristic->notify();
          delay(2000);
        }
      }
    }

    // Update previous positions
    previousXPosition = currentXPosition;
    previousYPosition = currentYPosition;
  } else {
    // Reset previous positions if no touch detected
    previousXPosition = 0;
    previousYPosition = 0;
  }
  
  delay(100);
}
